@@ -5,9 +5,10 @@
 
 use std::hash::{Hash, Hasher};
 use std::mem::size_of_val;
+use std::ops::Deref;
 use std::ptr;
-use std::rc::Rc;
-use std::sync::Arc;
+use std::rc::{Rc, Weak as WeakRc};
+use std::sync::{Arc, Weak as WeakArc};
 
 /// Type that can be compared in regard to identity (e.g. through address comparison) instead of
 /// equality (as done by [`PartialEq`])
@@ -53,6 +54,16 @@ impl<'a, T: ?Sized> Id for Rc<T> {
     }
 }
 
+/// [`std::rc::Weak`](WeakRc)s are identical if they share the same reference counter
+impl<'a, T: ?Sized> Id for WeakRc<T> {
+    fn same(&self, other: &Self) -> bool {
+        WeakRc::ptr_eq(self, &other)
+    }
+    fn hash<H: Hasher>(&self, state: &mut H) {
+        Hash::hash(&WeakRc::as_ptr(self), state)
+    }
+}
+
 /// `Arc`s are identical if they share the same reference counter
 impl<'a, T: ?Sized> Id for Arc<T> {
     fn same(&self, other: &Self) -> bool {
@@ -63,6 +74,16 @@ impl<'a, T: ?Sized> Id for Arc<T> {
     }
 }
 
+/// [`std::sync::Weak`](WeakArc)s are identical if they share the same reference counter
+impl<'a, T: ?Sized> Id for WeakArc<T> {
+    fn same(&self, other: &Self) -> bool {
+        WeakArc::ptr_eq(self, &other)
+    }
+    fn hash<H: Hasher>(&self, state: &mut H) {
+        Hash::hash(&WeakArc::as_ptr(self), state)
+    }
+}
+
 /// Newtype for references (or other types that implement [`Id`]) where [equality](PartialEq) is
 /// defined by the [`same`](Id::same) method of the [`Id`] trait
 ///
@@ -105,6 +126,62 @@ impl<T: ?Sized + Id> Hash for ById<T> {
     }
 }
 
+/// Similar to [`ById`], but uses [identity](Id) as defined by the dereferenced inner value
+///
+/// This can, for example, be used to wrap *references to* [`Rc`]s (or [`Arc`]s) while the identity
+/// depends on the *referenced* `Rc` or `Arc`).
+///
+/// # Example:
+///
+/// ```
+/// use std::rc::Rc;
+/// use refid::{ById, ByIdDeref};
+/// let rc1 = Rc::new(());
+/// let rc2 = Rc::new(());
+/// let rc3 = rc1.clone();
+/// assert_ne!(ByIdDeref(&rc1.clone()), ByIdDeref(&rc2.clone()));
+/// assert_eq!(ByIdDeref(&rc1.clone()), ByIdDeref(&rc3.clone()));
+/// // compare with:
+/// assert_ne!(ById(&rc1.clone()), ById(&rc2.clone()));
+/// assert_ne!(ById(&rc1.clone()), ById(&rc3.clone()));
+/// ```
+#[derive(Clone, Copy, Debug)]
+pub struct ByIdDeref<T: ?Sized>(
+    /// inner value
+    /// (which can be dereferenced to a reference or another type that implements [`Id`])
+    pub T,
+)
+where
+    T: Deref,
+    <T as Deref>::Target: Id;
+
+impl<T: ?Sized> PartialEq for ByIdDeref<T>
+where
+    T: Deref,
+    <T as Deref>::Target: Id,
+{
+    fn eq(&self, other: &Self) -> bool {
+        Id::same(&*self.0, &*other.0)
+    }
+}
+
+impl<T: ?Sized> Eq for ByIdDeref<T>
+where
+    T: Deref,
+    <T as Deref>::Target: Id,
+{
+}
+
+impl<T: ?Sized> Hash for ByIdDeref<T>
+where
+    T: Deref,
+    <T as Deref>::Target: Id,
+{
+    fn hash<H: Hasher>(&self, state: &mut H) {
+        Id::hash(&*self.0, state)
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -205,4 +282,15 @@ mod tests {
         let v3 = "X".to_string();
         assert_eq!(count_distinct(&vec![&v1, &v2, &v1, &v3]), 3);
     }
+    #[test]
+    fn test_by_id_deref() {
+        use std::rc::Rc;
+        let rc1 = Rc::new(());
+        let rc2 = Rc::new(());
+        let rc3 = rc1.clone();
+        assert_ne!(ById(&rc1.clone()), ById(&rc2.clone()));
+        assert_ne!(ById(&rc1.clone()), ById(&rc3.clone()));
+        assert_ne!(ByIdDeref(&rc1.clone()), ByIdDeref(&rc2.clone()));
+        assert_eq!(ByIdDeref(&rc1.clone()), ByIdDeref(&rc3.clone()));
+    }
 }
